#ifdef TEST
#include <stdio.h>
#include <ldap.h>
#endif


#define PASSWD_OK			0
#define PASSWD_BAD			1
#define PASSWD_COULD_NOT_CHECK		2

#define MANAGERDN	"cn=Directory Manager, o=Ace Industry, c=US"

int
password_is_ok( char *passwd )
{
	LDAP	*ld;
	int	lderr, rc;

	if ( (ld = ldap_init( "ldap.netscape.com", LDAP_PORT ))
	    == NULL ) {
		perror( "ldap_init" );
		return( PASSWD_COULD_NOT_CHECK );
	}

	lderr = ldap_simple_bind_s( ld, MANAGERDN, passwd );

	switch( lderr ) {
	case LDAP_SUCCESS:
		rc = PASSWD_OK;
		break;

	case LDAP_INVALID_CREDENTIALS:
	case LDAP_INAPPROPRIATE_AUTH:
		rc = PASSWD_BAD;
		break;
	default:	/* an unexpected error occurred */
		rc = PASSWD_COULD_NOT_CHECK;
		ldap_perror( ld, "ldap_simple_bind_s" );
	}

	ldap_unbind( ld );

	return( rc );
}


#ifdef TEST
main( int argc, char **argv )
{
	int		i, rc;

	if ( argc < 2 ) {
		fprintf( stderr, "%s: passwd\n", argv[ 0 ] );
		exit( 1 );
	}

	for ( i = 1; i < argc; ++i ) {
		printf( "password_is_ok( %s ):", argv[ i ] );
		rc = password_is_ok( argv[ i ] );
		printf( "\t%d (%s)\n\n", rc, ( rc == 0 ) ? "OK" :
		    ( rc == 1 ) ? "incorrect password" : "unable to check" );
	}

	exit( 0 );
}
#endif /* TEST */
