#ifdef TEST
#include <stdio.h>
#include <ldap.h>
#endif


int
entry_exists( LDAP *ld, char *dn )
{
	char		*attrs[ 2 ], *matched, *errmsg;
	int		lderr;
	LDAPMessage	*result;

	attrs[ 0 ] = "cn";
	attrs[ 1 ] = NULL;
	if ( (lderr = ldap_search_s( ld, dn, LDAP_SCOPE_BASE,
	    "(objectclass=*)", NULL, 0, &result )) == LDAP_SUCCESS ) {
		printf( "Entry %s exists.\n", dn );
		return( 1 );		/* entry exists */
	}

#ifdef LDAP_OPT_SIZELIMIT		/* only defined in Netscape SDK */
	(void)ldap_get_lderrno( ld, &matched, &errmsg );
#else
	matched = ld->ld_matched;
	errmsg = ld->ld_error;
#endif

	if ( lderr == LDAP_NO_SUCH_OBJECT ) {
		printf( "Entry %s does not exist.\n", dn );
		printf( "Matched DN:  %s\n", matched );
		return( 0 );
	}

	printf( "Could not check existence of %s\n", dn );
	printf( "because this error occurred: %s.\n",
	    ldap_err2string( lderr ));
	if ( errmsg != NULL ) {
		printf( "Server sent this additional info.: %s\n",
		    errmsg );
	}
	return( -1 );		/* error */
}


#ifdef TEST

#define HOST "belltower"

main( int argc, char **argv )
{
	int		i, rc;
	LDAP		*ld;

	if ( argc < 2 ) {
		fprintf( stderr, "%s: DN...\n", argv[ 0 ] );
		exit( 1 );
	}

	if ( (ld = ldap_init( HOST, LDAP_PORT )) == NULL ) {
		perror( HOST );
		exit( 1 );
	}

	for ( i = 1; i < argc; ++i ) {
		printf( "entry_exists( ld, %s ):\n", argv[ i ] );
		rc = entry_exists( ld, argv[ i ] );
		printf( "\t%d\n\n", rc );
	}

	exit( 0 );
}
#endif /* TEST */
