#include <stdio.h>
#include <ldap.h>

#define SEARCHBASE	"o=Netscape Communications Corp., c=US"

/*
 * include definition of print_entries_with_values() routine
 * from example X.Y above. 
 */
void
print_entries_with_values( LDAP *ld, LDAPMessage *result ) 
{
	LDAPMessage	*e;
	BerElement	*ber;
	char		*dn, *attr;
	char		**vals;
	int		i;

	for ( e = ldap_first_entry( ld, result ); e != NULL;
	   e = ldap_next_entry(ld, e ) ) {
		if ( (dn = ldap_get_dn( ld, e )) != NULL ) {
			printf( "dn: %s\n", dn );
			ldap_memfree( dn );
		}
		for ( attr = ldap_first_attribute( ld, e, &ber );
		   attr != NULL;
		   attr = ldap_next_attribute( ld, e, ber ) ) {
			if ( (vals = ldap_get_values( ld, e, attr ))
			   != NULL ) {
				for ( i = 0; vals[i] != NULL; i++ ) {
					printf( "%s: %s\n", attr, vals[i] );
				}
				ldap_value_free( vals );
			}
			ldap_memfree( attr );
		}
		printf( "\n" );
		if ( ber != NULL ) {
			ber_free( ber, 0 );
		}
	}
}

main()
{
	LDAP		*ld;
	LDAPMessage	*result;
	int		nentries, msgid, rc;

	if ( (ld = ldap_init( "ggood", LDAP_PORT ))
	   == NULL) {
		perror( "ldap_init" );
		exit( 1 );
	}

	if ( ldap_simple_bind_s( ld, NULL, NULL ) != LDAP_SUCCESS ) {
		ldap_perror( ld, "ldap_simple_bind_s" );
		exit( 1 );
	}

	if ( (msgid = ldap_search( ld, SEARCHBASE, LDAP_SCOPE_SUBTREE,
	   "(objectclass=*)", NULL, 0 )) == -1 ) {
		ldap_perror( ld, "ldap_search" );
		exit( 1 );
	}

	nentries = 0;
	while ( (rc = ldap_result( ld, msgid, 0, NULL, &result ))
	   == LDAP_RES_SEARCH_ENTRY ) {
		nentries++;
		print_entries_with_values( ld, result );
		ldap_msgfree( result );
	}
	if ( rc == -1) {
		ldap_perror( ld, "ldap_result" );
		exit( 1 );
	}

	printf( "Found %d entries at Ace Industry\n", nentries );
	ldap_unbind( ld );
	return( 0 );
}
