#ifdef TEST
#include <stdio.h>
#include <string.h>
#include <ldap.h>
#endif


char *
parentDN( char *dn )
{
	char	*parent, **rdns;
	int	i, len;

	if ( dn == NULL || *dn == '\0' ) {
		return( strdup( "" ));
	}

	if ( (rdns = ldap_explode_dn( dn, 0 )) == NULL ) {
		return( NULL );
	}

	/* calculate total length of parent DN */
	len = 1;	/* room for zero byte used to terminate strings */
	for ( i = 1; rdns[ i ] != NULL; ++i ) {
		len += ( 2 + strlen( rdns[ i ] ));  /* room for ", " and RDN */
	}
	/* allocate space to hold the parent DN */
	if ( (parent = (char *) malloc( len )) == NULL ) {
		ldap_value_free( rdns );
		return( NULL );
	}

	/* construct parent DN from RDNs */
	*parent = '\0';
	for ( i = 1; rdns[ i ] != NULL; ++i ) {
		if ( i > 1 ) {
			strcat( parent, ", " );
		}
		strcat( parent, rdns[ i ] );
	}

	ldap_value_free( rdns );
	return( parent );
}


#ifdef TEST
main( int argc, char **argv )
{
	int		i, rc;
	LDAP		*ld;

	if ( argc < 2 ) {
		fprintf( stderr, "%s: DN...\n", argv[ 0 ] );
		exit( 1 );
	}

	for ( i = 1; i < argc; ++i ) {
		printf( "parentDN( %s ): %s\n", argv[ i ],
		    parentDN( argv[ i ] ));
	}

	exit( 0 );
}
#endif /* TEST */
