#include <stdio.h>
#include <ldap.h>
#include <ctype.h>

void write_escaped( char *s );

main()
{
	char		url[ 2048 ];
	int		err;
	LDAPURLDesc	*ludp;

	while ( gets( url ) != NULL ) {
		if ( !ldap_is_ldap_url( url )) {
			puts( url );	/* echo non-LDAP URLs unchanged */
			continue;
		}

		if (( err = ldap_url_parse( url, &ludp )) != 0 ) {
			fprintf( stderr, "Unable to parse LDAP URL %s (error %d)\n",
			    url, err );
			exit( 1 );
		}

		fputs( "ldap://", stdout );
		write_escaped( ludp->lud_host );
		if ( ludp->lud_port != 0 ) {
			printf( ":%d", ludp->lud_port );
		}
		putchar( '/' );

		write_escaped( ludp->lud_dn );
		fputs( "?cn,drink", stdout );

		if ( ludp->lud_scope == LDAP_SCOPE_SUBTREE) {
			fputs( "?sub", stdout );
		} else if ( ludp->lud_scope == LDAP_SCOPE_ONELEVEL ) {
			fputs( "?one", stdout );
		} else if ( ludp->lud_filter != NULL ) {
			fputs( "?base", stdout );
		}

		if ( ludp->lud_filter != NULL ) {
			putchar( '?' );
			write_escaped( ludp->lud_filter );
		}
		putchar( '\n' );

		ldap_free_urldesc( ludp );
	}

	exit( 0 );

}

#define IS_URL_SAFE( c )	( isalpha(c) || isdigit(c) || (c) == '.' || \
					(c) == '=' || (c) == ',' )

void
write_escaped( char *s )
{
	if ( s == NULL ) {
		return;
	}

	while ( *s != '\0' ) {
		if ( IS_URL_SAFE( *s )) {
			putchar( *s );
		} else {
			printf( "%%%2x", *s );
		}
		++s;
	}
}

