#!/usr/local/bin/perl
#
# lookup -- Perl 4/5 script
#
# A simple HTML interface to an LDAP directory 
#
# Notes:
#		should handle continued lines produced by ldapsearch
#		should display multiple values if present
	
# location of ldapsearch command
$ldapsearch = "/usr/local-local/bin/ldapsearch";
	
# begin CGI output 
print("Content-type: text/html\n\n");
print( "<HTML><HEAD><TITLE>LDAP Search Results</TITLE></HEAD><BODY>\n" );
print( "<CENTER><H2>LDAP Search Results</H2></CENTER>\n" );
	
# build an associative array from the values contained in the URL's
# query string
@cgiQueryPairs = split( "&", $ENV{ 'QUERY_STRING' } );
foreach $p ( @cgiQueryPairs ) {
        ( $var, $val ) = split( "=", $p );
        $val =~ s/\+/ /g;                       # translate '+' to space
        $val =~ s/%(..)/pack("c",hex($1))/ge;   # undo '%HH' URL-encoding
        $cgiValues{ "$var" } = "$val";
}
	
# pull out required form variables
$searchString = $cgiValues{ "searchstring" };
if ( $searchString eq "" ) {
        &formVarMissingError( "searchstring" );
}
	
$directory = $cgiValues{ "directory" };
if ( $directory eq "" ) {
        &formVarMissingError( "directory" );
}
( $var, $base ) = split( '/', $directory );
( $host, $port ) = split( ':', $var );
if ( $port eq "" ) {
        $port = "389";
}
	
$tmp = $cgiValues{ "attrlist" };
if ( $tmp eq "" ) {
        &formVarMissingError( "attrlist" );
}
@attrList = split( "/", $tmp );
	
# pull out optional form variables
$sortAttr = $cgiValues{ "sortattr" };
	
$tmp = $cgiValues{ "displaynames" };
if ( $tmp eq "" ) {
        @displayNames = @attrList;
} else {
        @displayNames = split( "/", $tmp );
}
for ( $i=0; $i < scalar( @attrList ); $i++ ) {
        $attr2Name{ $attrList[ $i ] } = $displayNames[ $i ];
}
	
# build ldapsearch command
$searchcmd = join( " ", ( $ldapsearch, "-h $host -p $port",
        "-b \"$base\"", "-S \"$sortAttr\"",
        "\"(|(cn=$searchString)(sn=$searchString)(uid=$searchString))\"",
        @attrList ));
	
# fire off ldapsearch command
if ( !open( SRCHOUT, "$searchcmd|" )) {
        &reportError( "Unable to execute search" );
}
	
$count = 0;
$inEntry = 0;
	
while ( $line = <SRCHOUT> ) {
        chop( $line );
        if ( length( $line ) == 0 ) {
                &displayEntry( $sortAttr, $attrVals );
                $inEntry = 0;
                $count++;
        }
        ($tag,$val) = split( ':', $line );
        if ( ! $inEntry && $tag eq "dn" ) {
                $inEntry = 1;
        }
        $attrVals{ "$tag" } = "$val";
}
	
if ( $inEntry ) {
        &displayEntry( $sortAttr, $attrVals );
        $count++;
}
	
print( "<HR><H3>Found ",  $count, " entries that matched \"" , $searchString,
        "\".</H3>\n" );
	
$rc = close( SRCHOUT );
	
print( "</BODY></HTML>\n" );
	
exit( 0 );
	
	
# subroutine to report an error and exit
sub
reportError {
        print( "<B>Error:<B> ", @_, "\n</BODY></HTML>\n" );
        exit( 0 );
}
	
# report "required form variable missing" error and exit
sub
formVarMissingError {
        &reportError( join( " ", ( "Missing required form variable", @_ )));
}
	
	
# subroutine to display an LDAP entry as HTML
sub
displayEntry {
        $attrVals = @_[0];
        $sortAttr = @_[1];
	
        print( "<HR>\n<H3>", $attrVals{ "dn" }, "</H3>\n" );
        print( "<DL>\n" );
        if ( $sortAttr ne "" ) {
                &displayAttr( $attrVals, $sortAttr  );
        }
	
        foreach $attr ( @attrList ) {
                if ( $attr ne $sortAttr ) {
                         &displayAttr( $attrVals, $attr );
                }
        }
        print( "</DL>\n" );
	
# remove the evidence
        foreach $key ( keys(%attrVals)) {
                delete( $attrVals{ $key } );
        }
}
	
	
sub
displayAttr {
        $attrVals = @_[0];
        $attr = @_[1];
        $displayName = $attr2Name{ $attr };
	
        $val = $attrVals{ $attr };
        if ( $val ne "" ) {
                print( "<DT>", $displayName, ":\n<DD>" );
                if ( $attr eq "mail" ) {
                        print( "<A HREF=\"mailto:$val\">$val</A>" );
                } else {
                         print( $val );
                }
                print( "</DT>\n" );
        }
}
